/*
	`xsvif.c'
	XSV Board loader interface routines

	20000516, Andrew Bardsley
*/

#include "xsvif.h"
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>

unsigned char XSV_CheckSum = 0;
int XSV_SerialPortFD = 0;
struct termios XSV_OriginalPortSettings;

void (* XSV_Cleanup) (void) = NULL;

/* XSV_DefaultCleanup: default cleanup function set by XSV_Startup */
void XSV_DefaultCleanup (void)
{
	if (XSV_SerialPortFD) XSV_CloseSerialPort (XSV_SerialPortFD);
	exit (EXIT_SUCCESS);
}

/* XSV_Error: simple fatal error report */
void XSV_Error (char *str)
{
	fprintf (stderr, "XSV Loader: %s\n", str);
	if (XSV_Cleanup) XSV_Cleanup ();
	exit (EXIT_FAILURE);
}

/* XSV_OpenSerialPort: open the named serial port and return an
	open bidirectional file descriptor for raw I/O on that port.
	Sets up the port for: 115200 bps, 8 bits, no parity, 1 stop */
int XSV_OpenSerialPort (char *port)
{
    int portFD;
    struct termios serialSettings;

	portFD = open (port, O_RDWR | O_NOCTTY | O_NDELAY);
    if (portFD < 0) { XSV_Error ("couldn't open serial port"); }
	fcntl (portFD, F_SETFL, 0);

	if (tcgetattr (portFD, &XSV_OriginalPortSettings) == -1)
		XSV_Error ("failed to serial port attributes");

/*
	serialSettings = XSV_OriginalPortSettings;
*/

	memset ((void *) &serialSettings, 0, sizeof (struct termios));
/*
	serialSettings.c_cflag &= ~CSIZE;
	serialSettings.c_cflag &= ~PARENB;
	serialSettings.c_cflag &= ~CSTOPB;
	serialSettings.c_cflag &= ~CRTSCTS;
*/
	serialSettings.c_cflag |= CREAD | CLOCAL | CS8 | PARENB | PARODD;
/*
	serialSettings.c_iflag &= ~(IXON | IXOFF | IXANY);

	serialSettings.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);

*/

	serialSettings.c_cc[VMIN] = 0;
	serialSettings.c_cc[VTIME] = 4;

	cfsetospeed (&serialSettings, B2400);
	cfsetispeed (&serialSettings, B2400);
    
	if (tcsetattr (portFD, TCSANOW, &serialSettings))
		XSV_Error ("failed to update serial port attributes");

	return portFD;
}

/* XSV_Startup: run OpenSerialPort, setup cleanup function and signal handling */
void XSV_Startup (char *port)
{
	struct sigaction cleanup;

	XSV_Cleanup = XSV_DefaultCleanup;

	cleanup.sa_handler = (void (*)()) XSV_Cleanup;
	sigemptyset (&cleanup.sa_mask);
	cleanup.sa_flags = 0;	

	sigaction (SIGINT, &cleanup, NULL);
	sigaction (SIGHUP, &cleanup, NULL);
	sigaction (SIGTERM, &cleanup, NULL);
	sigaction (SIGSEGV, &cleanup, NULL);
	sigaction (SIGBUS, &cleanup, NULL);

	XSV_SerialPortFD = XSV_OpenSerialPort (port);
}

/* XSV_CloseSerialPort: close the port opened on fildes `portFD'
	and restore the serial port's attributes */
/* XSV_Error: simple fatal error report */
void XSV_CloseSerialPort (int portFD)
{
    tcsetattr (portFD, TCSANOW, &XSV_OriginalPortSettings);
	close (portFD);
}

/* XSV_PutData: write some data to the board, handle errors */
void XSV_PutData (int portFD, char *buffer, int bufferSize)
{
	int writeRet = write (portFD, buffer, bufferSize);
	/*printf("%x\n",*buffer);*/
	
	if (writeRet == -1)
		XSV_Error ("error writing data data to board");
	else {
		int i;
		for (i=10;i;i--);

		for (i = 0; i < writeRet; i++) { XSV_CheckSum ^= buffer[i];}
	     }
}

/* XSV_GetAck: get acknowledgement from board, handle timeout */
char XSV_GetAck (int portFD)
{
	char ack;
	int readRet = read (portFD, &ack, 1);

	if (readRet == -1)
		XSV_Error ("error receiving board acknowledgement");
	else if (readRet == 0)
		XSV_Error ("timeout receiving board acknowledgement");

	return ack;
}

